/*
 *  @file   SharedMemoryMgr.c
 *
 *  @brief   Implements  wrapper functions over Heap  APIs and Shared region
 *           address translation API. Provides API to work on shared memory
 *           over which heap and shared region is created. instance  create takes
 *           heap handle created on shared memory for allocations.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



/*-------------------------    XDC  specific includes ----------------------  */
#include <xdc/std.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Startup.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/IHeap.h>

/*-------------------------    Generic includes  ---------------------------- */
#include <string.h>

/*-------------------------    IPC module specific includes ------------------*/
#include <ti/sdo/ipc/_SharedRegion.h>

#include <ti/syslink/inc/SyslinkMemMgr_errBase.h>

#include "package/internal/SharedMemoryMgr.xdc.h"




#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  External APIs
 *  ============================================================================
 */

/*
 *  ======== SharedMemoryMgr_Params_init ========
 *  Function to get the default parameters of the SharedMemory SyslinkMemMgr.
 *
 *  This function can be called by the application to get their
 *  default parameters.
 *
 */

Int SharedMemoryMgr_Instance_init(SharedMemoryMgr_Object       *obj,
                               const SharedMemoryMgr_Params *params,
                               Error_Block               *eb)
{

    Int32 status = MEMMGR_SUCCESS;

    //Assert_isTrue(obj != NULL, SharedMemoryMgr_A_nullArgument);
    obj->heapHandle = params->heapHandle;

    return(status);
}

Void SharedMemoryMgr_Instance_finalize(SharedMemoryMgr_Object *obj,
                                     Int                   status)
{
    //Assert_isTrue(obj != NULL, SharedMemoryMgr_A_nullArgument);
    obj->heapHandle = NULL;

}

/* Function to allocate memory from the SyslinkMemMgr */
Ptr
SharedMemoryMgr_alloc (SharedMemoryMgr_Object *obj,
                       Ptr                     allocParams)
{
    Ptr   buf                                    = NULL;
    SharedMemoryMgr_allocParams  * instAllocParams =
                                      (SharedMemoryMgr_allocParams  *)allocParams;
    Error_Block eb;

    Error_init (&eb);

    if (obj != NULL) {
        buf = Memory_alloc(obj->heapHandle,
                           instAllocParams->size,
                           instAllocParams->align,
                           &eb);
    }

    return (buf);
}

/* Function to allocate memory from the SyslinkMemMgr */
Int32
SharedMemoryMgr_free (SharedMemoryMgr_Object *obj,
                      Ptr                     freeParams)
{
    Int32 status = MEMMGR_SUCCESS;
    SharedMemoryMgr_freeParams  * instFreeParams =
                                      (SharedMemoryMgr_freeParams  *)freeParams;
    if (obj == NULL) {
        status = MEMMGR_E_FAIL;
    }
    else {
        Memory_free((IHeap_Handle)obj->heapHandle,
                     instFreeParams->ptr,
                     instFreeParams->size);
    }

    return (status);
}

/* Function to allocate memory from the SyslinkMemMgr.Calls sharedregion API to do
 * address translations
 */
Ptr
SharedMemoryMgr_translate (SharedMemoryMgr_Object   *obj,
                           Ptr                       srcAddr,
                           SharedMemoryMgr_AddrType  srcAddrType,
                           SharedMemoryMgr_AddrType  desAddrType)
{
    Ptr       dstAddr = NULL;
    Int32     index;

    if (obj != NULL) {
        if (srcAddrType == desAddrType) {
            /* Return NULL if both addr types are same */
            dstAddr = srcAddr;
        }
        else {
            switch (srcAddrType)
            {
                case SharedMemoryMgr_AddrType_Virtual:
                {
                    if (desAddrType == SharedMemoryMgr_AddrType_Portable) {
                        index = SharedRegion_getId (
                                                (Ptr)srcAddr);
                        dstAddr = (Ptr)SharedRegion_getSRPtr ((Ptr)srcAddr,
                                                         index);
                        if (dstAddr == (Ptr)ti_sdo_ipc_SharedRegion_INVALIDSRPTR) {
                            dstAddr = NULL;
                        }

                    }
                }
                break;

                case SharedMemoryMgr_AddrType_Portable:
                {
                    if (desAddrType == SharedMemoryMgr_AddrType_Virtual) {
                        dstAddr = SharedRegion_getPtr ((SharedRegion_SRPtr)srcAddr);
                    }
                }
                break;

                default:
                    dstAddr = NULL;
            }
        }
    }

    return (dstAddr);

}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */






